@@ -29,7 +29,7 @@
  * @copyright   2007 Ryusuke SEKIYAMA
  * @copyright   2009 Keynetics Inc
  * @license     http://www.opensource.org/licenses/mit-license.php  MIT License
- * @version     SVN: $Id: parse.c 323565 2012-02-26 17:06:44Z bd808 $
+ * @version     SVN: $Id$
  */
 
 
@@ -429,8 +429,10 @@ static zval *handle_mapping(parser_state_t *state TSRMLS_DC)
 		}
 
 		/* check for '<<' and handle merge */
-		if (IS_NOT_QUOTED_OR_TAG_IS(key_event, YAML_MERGE_TAG) && 
-				STR_EQ("<<", key_str)) {
+		if (key_event.type == YAML_SCALAR_EVENT &&
+				IS_NOT_QUOTED_OR_TAG_IS(key_event, YAML_MERGE_TAG) &&
+				STR_EQ("<<", key_str) &&
+				Z_TYPE_P(value) == IS_ARRAY) {
 			/* zend_hash_merge */
 			/*
 			 * value is either a single ref or a simple array of refs
@@ -972,7 +974,11 @@ static char *convert_to_char(zval *zv TSRMLS_DC)
 			php_var_serialize(&buf, &zv, &var_hash TSRMLS_CC);
 			PHP_VAR_SERIALIZE_DESTROY(var_hash);
 
-			str = buf.c;
+			if (buf.c) {
+				str = estrndup(buf.c, buf.len);
+			} else {
+				str = NULL;
+			}
 		}
 		break;
 	}
