@@ -29,7 +29,7 @@
  * @copyright   2007 Ryusuke SEKIYAMA
  * @copyright   2009 Keynetics Inc
  * @license     http://www.opensource.org/licenses/mit-license.php  MIT License
- * @version     SVN: $Id: yaml.c 324514 2012-03-24 23:57:50Z bd808 $
+ * @version     SVN: $Id$
  */
 
 #include "php_yaml.h"
@@ -176,7 +176,7 @@ zend_module_entry yaml_module_entry = {	/* {{{ */
 	NULL, /* RINIT */
 	NULL, /* RSHUTDOWN */
 	PHP_MINFO(yaml),
-	PHP_YAML_MODULE_VERSION,
+	PHP_YAML_VERSION,
 #if ZEND_EXTENSION_API_NO >= 220060519
 	PHP_MODULE_GLOBALS(yaml),
 	PHP_GINIT(yaml),
@@ -280,7 +280,7 @@ PHP_MINFO_FUNCTION(yaml)
 {
 	php_info_print_table_start();
 	php_info_print_table_row(2, "LibYAML Support", "enabled");
-	php_info_print_table_row(2, "Module Version", PHP_YAML_MODULE_VERSION);
+	php_info_print_table_row(2, "Module Version", PHP_YAML_VERSION);
 	php_info_print_table_row(2, "LibYAML Version", yaml_get_version_string());
 	php_info_print_table_end();
 
@@ -774,9 +774,6 @@ PHP_FUNCTION(yaml_emit_file)
 	}
 #endif
 
-	php_error_docref(NULL TSRMLS_CC, E_WARNING, "not yet implemented");
-	RETURN_FALSE;
-
 	if (NULL == (stream = php_stream_open_wrapper(filename, "wb",
 			IGNORE_URL | ENFORCE_SAFE_MODE | REPORT_ERRORS | STREAM_WILL_CAST,
 			NULL))) {
